package de.hampelratte.yaggui;

import java.io.PrintStream;
import java.text.DecimalFormat;

/**
 * Manages all output. The Logger supports logging levels. So you are able to
 * set a threshold value for logging. The order: OFF VERBOSE DEBUG INFO WARN
 * ERROR
 */
public class Logger {
    private static Logger logger;

    public static final int OFF = 0;

    public static final int VERBOSE = 1;

    public static final int DEBUG = 2;

    public static final int INFO = 3;

    public static final int WARN = 4;

    public static final int ERROR = 5;

    public static final String[] levelname = { "OFF  ", "VERBOSE", "DEBUG  ",
            "INFO   ", "WARN   ", "ERROR  " };

    private static PrintStream out = System.out;

    private static int level = DEBUG;

    private int nameLength = 50;

    /**
     * %l = level %n = class.method %s = source (linenumber);
     */
    private String pattern = "%l %n - ";

    private Logger() {
    }

    public static Logger getLogger() {
        if (logger == null)
            logger = new Logger();
        return logger;
    }

    /**
     * Sets the level of the logger. All levels below the new one won't be
     * logged anymore.
     * 
     * @param l
     *            the new level
     */
    public static void setLevel(int l) {
        getLogger().debug("Setting logging level to: " + l);
        level = l;
    }

    /**
     * Sets the OutputStream the logger will write the output to. Think of
     * System.out, a file or a socket.
     * 
     * @param o
     *            the new PrintStream
     */
    public static void setOutputStream(PrintStream o) {
        out = o;
    }

    /**
     * Logs an object with level DEBUG
     */
    public void debug(Object o) {
        log(o, DEBUG);
    }

    /**
     * Logs an object with level INFO
     */
    public void info(Object o) {
        log(o, INFO);
    }

    /**
     * Logs an object with level WARN
     */
    public void warn(Object o) {
        log(o, WARN);
    }

    /**
     * Logs an object with level ERROR
     */
    public void error(Object o) {
        log(o, ERROR);
    }

    /**
     * Logs an object with level FATAL
     */
    public void verbose(Object o) {
        log(o, VERBOSE);
    }

    /**
     * Logs an object with a given level
     */
    private void log(Object o, int level) {
        Throwable t = new Throwable();
        StackTraceElement[] stacktrace = t.getStackTrace();
        StackTraceElement source = stacktrace[2];

        String className = source.getClassName();
        String methodName = source.getMethodName();
        Integer l = new Integer(source.getLineNumber());
        DecimalFormat df = new DecimalFormat("0000");
        String line = df.format(l);

        String name = className + "." + methodName;
        if (name.length() > nameLength) {
            name = name.substring(0, nameLength);
        } else if (name.length() < nameLength) {
            int count = nameLength - name.length();
            for (int i = 0; i < count; i++) {
                name += " ";
            }
        }

        String log = "";
        if (Logger.level > OFF && Logger.level <= level) {
            log = pattern.replaceAll("%l", levelname[level]).replaceAll("%n",
                    name).replaceAll("%s", line);
            log += o;
            out.println(log);
        }

        /*
         * if(level >= Logger.ERROR) {
         * javax.swing.JOptionPane.showMessageDialog(Main.getGUI(),
         * o.toString(), name, javax.swing.JOptionPane.ERROR_MESSAGE); }
         */
    }
}