package de.hampelratte.yaggui.gui;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

import de.hampelratte.yaggui.Logger;
import de.hampelratte.yaggui.Main;
import de.hampelratte.yaggui.StatsCaller;

public class MainFrame extends JFrame implements ActionListener {
    private static Logger logger = Logger.getLogger();

    private JPanel contentPane;

    private CardLayout cards = new CardLayout();

    private JToolBar toolbar = new JToolBar();

    private JPanel mainPanel = new JPanel();

    private JButton search = new JButton();

    private JButton transfer = new JButton();

    private JButton files = new JButton();

    private JButton peers = new JButton();

    private JButton settings = new JButton();

    private JButton startgift = new JButton();

    private JButton killgift = new JButton();

    private JPanel statusbar = new JPanel();

    private JLabel status = new JLabel(" ");

    private FilePanel filePanel = new FilePanel(mainPanel);

    private SearchPanel searchPanel = new SearchPanel(mainPanel, this);

    private TransferPanel transferPanel = new TransferPanel(mainPanel);

    private SettingsPanel settingsPanel = new SettingsPanel(mainPanel);

    private boolean sharesLoaded = false;

    boolean connected = false;

    public MainFrame() {
        init();
        new StatsCaller().start();
    }

    private void init() {
        contentPane = (JPanel) this.getContentPane();
        contentPane.setLayout(new BorderLayout());

        // create buttons for toolbar
        search.setText("Search");
        search.setIcon(Main.searchIcon);
        search.setVerticalTextPosition(SwingConstants.BOTTOM);
        search.setHorizontalTextPosition(SwingConstants.CENTER);
        search.setMargin(new Insets(5, 10, 5, 10));
        search.setFont(new Font("SansSerif", Font.PLAIN, 11));
        //search.setBorderPainted(false);
        search.setRolloverEnabled(true);
        search.setEnabled(false);
        transfer.setText("Transfer");
        transfer.setIcon(Main.transferIcon);
        transfer.setVerticalTextPosition(SwingConstants.BOTTOM);
        transfer.setHorizontalTextPosition(SwingConstants.CENTER);
        transfer.setMargin(new Insets(5, 10, 5, 10));
        transfer.setFont(new Font("SansSerif", Font.PLAIN, 11));
        transfer.setEnabled(false);
        files.setText("Shared Files");
        files.setIcon(Main.sharedIcon);
        files.setVerticalTextPosition(SwingConstants.BOTTOM);
        files.setHorizontalTextPosition(SwingConstants.CENTER);
        files.setMargin(new Insets(5, 10, 5, 10));
        files.setFont(new Font("SansSerif", Font.PLAIN, 11));
        files.setEnabled(false);
        peers.setText("Connect");
        peers.setIcon(Main.peersIcon);
        peers.setVerticalTextPosition(SwingConstants.BOTTOM);
        peers.setHorizontalTextPosition(SwingConstants.CENTER);
        peers.setMargin(new Insets(5, 10, 5, 10));
        peers.setFont(new Font("SansSerif", Font.PLAIN, 11));
        settings.setText("Settings");
        settings.setIcon(Main.settingsIcon);
        settings.setVerticalTextPosition(SwingConstants.BOTTOM);
        settings.setHorizontalTextPosition(SwingConstants.CENTER);
        settings.setMargin(new Insets(5, 10, 5, 10));
        settings.setFont(new Font("SansSerif", Font.PLAIN, 11));

        startgift.setText("Start giFT");
        startgift.setIcon(Main.startIcon);
        startgift.setVerticalTextPosition(SwingConstants.BOTTOM);
        startgift.setHorizontalTextPosition(SwingConstants.CENTER);
        startgift.setMargin(new Insets(5, 10, 5, 10));
        startgift.setFont(new Font("SansSerif", Font.PLAIN, 11));

        killgift.setText("Kill giFT");
        killgift.setIcon(Main.killIcon);
        killgift.setVerticalTextPosition(SwingConstants.BOTTOM);
        killgift.setHorizontalTextPosition(SwingConstants.CENTER);
        killgift.setMargin(new Insets(5, 10, 5, 10));
        killgift.setFont(new Font("SansSerif", Font.PLAIN, 11));
        killgift.setEnabled(false);

        // add buttons to toolbar
        toolbar.addSeparator();
        toolbar.add(peers);
        toolbar.addSeparator();
        toolbar.add(search);
        toolbar.addSeparator();
        toolbar.add(transfer);
        toolbar.addSeparator();
        toolbar.add(files);
        toolbar.addSeparator();
        toolbar.add(settings);
        toolbar.addSeparator();
        JPanel panel = new JPanel();
        panel.setLayout(new FlowLayout(FlowLayout.RIGHT));
        panel.add(startgift);
        panel.add(killgift);
        toolbar.add(panel);
        toolbar.addSeparator();
        toolbar.setRollover(true);
        toolbar.setFloatable(false);

        // init main panel
        mainPanel.setLayout(cards);
        mainPanel.add(filePanel, "filepanel");
        mainPanel.add(searchPanel, "searchpanel");
        mainPanel.add(transferPanel, "transferpanel");
        mainPanel.add(settingsPanel, "settingspanel");
        cards.show(mainPanel, "settingspanel");

        // init statusbar
        statusbar.setLayout(new FlowLayout(FlowLayout.LEFT));
        statusbar.setBackground(UIManager.getColor("Statusbar.background"));
        statusbar.add(status);

        // add components to contentPane
        contentPane.add(toolbar, BorderLayout.NORTH);
        contentPane.add(mainPanel, BorderLayout.CENTER);
        contentPane.add(statusbar, BorderLayout.SOUTH);

        // add listeners
        search.addActionListener(this);
        transfer.addActionListener(this);
        files.addActionListener(this);
        peers.addActionListener(this);
        settings.addActionListener(this);
        startgift.addActionListener(this);
        killgift.addActionListener(this);

        // set frame properties
        if (Main.confTable.get("width") != null
                && Main.confTable.get("height") != null) {
            this.setSize(new Double((String) Main.confTable.get("width"))
                    .intValue(), new Double((String) Main.confTable
                    .get("height")).intValue());
        } else {
            this.setSize(800, 600);
        }
        if (Main.confTable.get("x") != null && Main.confTable.get("x") != null) {
            this.setLocation(new Double((String) Main.confTable.get("x"))
                    .intValue(), new Double((String) Main.confTable.get("y"))
                    .intValue());
        } else {
            this.setLocation(0, 0);
        }
        this.setTitle("yaggui " + Main.VERSION + " [disconnected]");
        this.status.setIcon(Main.disconnectedIcon);
        this.status.setText("Offline");
        this.status.setToolTipText("Connection status: offline");
        this.setDefaultCloseOperation(EXIT_ON_CLOSE);
        this.setIconImage(Main.titleIcon.getImage());
        this.setVisible(true);
        if (Main.confTable.get("extendedState") != null) {
            this.setExtendedState(new Integer((String) Main.confTable
                    .get("extendedState")).intValue());
        }

        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                Main.confTable.put("extendedState", new Integer(
                        getExtendedState()));
                double width = getSize().getWidth();
                double height = getSize().getHeight();
                Main.confTable.put("width", new Double(width));
                Main.confTable.put("height", new Double(height));
                double x = getLocation().getX();
                double y = getLocation().getY();
                Main.confTable.put("x", new Double(x));
                Main.confTable.put("y", new Double(y));
                Main.writeConf();
                try {
                    Main.com.detach();
                } catch (Exception e) {
                }
                System.exit(0);
            }
        });
    }

    public boolean isConnected() {
        return this.connected;
    }

    public void setConnected() {
        connected = !connected;
        peers.setText("Disconnect");
        peers.setIcon(Main.disconnectIcon);
        files.setEnabled(true);
        transfer.setEnabled(true);
        search.setEnabled(true);
        killgift.setEnabled(true);
        this.status.setIcon(Main.connectedIcon);
        this.status.setText("Online");
        this.setTitle("yaggui " + Main.VERSION + " [connected]");
        cards.show(mainPanel, "searchpanel");
        while (Main.com == null) {
            try {
                Thread.sleep(10);
            } catch (Exception e) {
            }
        }
        getDataFromGiftd();
    }

    public void disconnect() {
        connected = false;
        sharesLoaded = false;
        Main.com.detach();
        peers.setText("Connect");
        peers.setIcon(Main.peersIcon);
        this.setTitle("yaggui " + Main.VERSION + " [disconnected]");
        this.status.setIcon(Main.disconnectedIcon);
        this.status.setText("Offline");
        this.status.setToolTipText("Connection status: offline");
        filePanel.resetTable();
        searchPanel.setAllEnded();
        transferPanel.resetTables();
        files.setEnabled(false);
        transfer.setEnabled(false);
        search.setEnabled(false);
        killgift.setEnabled(false);
    }

    private void getDataFromGiftd() {
        Main.com.stats();
    }

    public FilePanel getFilePanel() {
        return filePanel;
    }

    public SearchPanel getSearchPanel() {
        return searchPanel;
    }

    public TransferPanel getTransferPanel() {
        return transferPanel;
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == files) {
            cards.show(mainPanel, "filepanel");
            if (!sharesLoaded && connected) {
                sharesLoaded = true;
                Main.com.getShares();
            }
        } else if (e.getSource() == search) {
            cards.show(mainPanel, "searchpanel");
        } else if (e.getSource() == transfer) {
            cards.show(mainPanel, "transferpanel");
        } else if (e.getSource() == peers) {
            if (!connected) {
                Main.com.attach();
            } else {
                disconnect();
            }
        } else if (e.getSource() == settings) {
            cards.show(mainPanel, "settingspanel");
        } else if (e.getSource() == startgift) {
            try {
                Runtime rt = Runtime.getRuntime();
                rt.exec((String) Main.confTable.get("giftd"));
            } catch (IOException ioe) {
                logger.error("Couldn't start giftd\n\t" + ioe);
            }
        } else if (e.getSource() == killgift) {
            if (connected) {
                Main.com.shutdown();
                disconnect();
            } else {
                JOptionPane.showMessageDialog(this,
                        "yaggui is not connected to giftd", "Error",
                        JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    public void showPanel(String panel) {
        try {
            cards.show(mainPanel, panel);
        } catch (Exception e) {
        }
    }

    public void setStatusText(String status) {
        this.status.setText(status);
    }

    public void setStatusToolTip(String tt) {
        this.status.setToolTipText(tt);
    }
}