package de.hampelratte.yaggui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import de.hampelratte.yaggui.Logger;
import de.hampelratte.yaggui.Main;
import de.hampelratte.yaggui.Result;
import de.hampelratte.yaggui.Subcommand;
import de.hampelratte.yaggui.gui.sortabletable.SortableTable;
import de.hampelratte.yaggui.gui.sortabletable.SortableTableHeaderRenderer;
import de.hampelratte.yaggui.gui.sortabletable.SortableTableModel;

class ResultPanel extends JPanel implements ActionListener {
    private Logger logger = Logger.getLogger();

    private boolean ended = false;

    private SortableTable table;

    private SortableTableModel model;

    private JPanel statusbar = new JPanel();

    private JLabel status = new JLabel(" ");

    private JProgressBar progress = new JProgressBar(JProgressBar.HORIZONTAL);

    private JPopupMenu popup = new JPopupMenu();

    private JMenuItem popDownload = new JMenuItem("Download");

    private int sessionID;

    private int hits = 0;

    private ShareTableCellRenderer stcr = new ShareTableCellRenderer();

    private JButton stopButton = new JButton(Main.stopIcon);

    private String title = "";

    ResultPanel(int sessionID, String title) {
        this.title = title;
        this.sessionID = sessionID;
        setLayout(new BorderLayout());
        Object[] columnNames = { "File", "Sources", "Availability", "Size",
                "Bitrate", "Duration", "Type", "Resolution", "Hash" };
        int[] columnWidth = { 300, 65, 80, 80, 80, 80, 100, 80, 340 };

        table = new SortableTable();
        model = new SortableTableModel(9, table);
        table.setModel(model);
        table.setShowGrid(false);
        table.setRowHeight(20);
        table.setRowMargin(0);
        table.setShowVerticalLines(false);
        table.setShowHorizontalLines(false);
        model.setColumnIdentifiers(columnNames);

        for (int i = model.getColumnCount() - 1; i >= 0; i--) {
            DefaultTableColumnModel tcm = (DefaultTableColumnModel) table
                    .getColumnModel();
            TableColumn tc = tcm.getColumn(i);
            tc.setCellRenderer(stcr);
        }
        SortableTableHeaderRenderer renderer = new SortableTableHeaderRenderer();
        TableColumnModel tcmodel = table.getColumnModel();
        int n = columnNames.length;
        for (int i = 0; i < n; i++) {
            tcmodel.getColumn(i).setHeaderRenderer(renderer);
            tcmodel.getColumn(i).setPreferredWidth(columnWidth[i]);
        }
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        JScrollPane scrollpane = new JScrollPane(table);
        scrollpane.setBorder(BorderFactory.createEmptyBorder());
        //scrollpane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        add(scrollpane, BorderLayout.CENTER);
        table.init();

        statusbar.setLayout(new FlowLayout(FlowLayout.LEFT));
        statusbar.setBackground(UIManager.getColor("Statusbar.background"));
        statusbar.add(status);
        progress.setIndeterminate(true);
        progress.setStringPainted(true);
        progress.setString("running");
        progress.setPreferredSize(new Dimension(100, 18));
        status.setText("<html><font color=red>" + hits
                + " hits&nbsp;&nbsp;</font></html>");
        statusbar.add(progress);
        statusbar.add(stopButton);
        stopButton.addActionListener(this);
        stopButton.setPreferredSize(new Dimension(24, 24));
        stopButton.setBorderPainted(false);
        stopButton.setBackground(statusbar.getBackground());

        add(statusbar, BorderLayout.SOUTH);

        popDownload.addActionListener(this);
        popup.add(popDownload);

        table.addMouseListener(new MouseAdapter() {
            public void mouseReleased(MouseEvent e) {
                if (e.isShiftDown() || e.isControlDown() || e.isAltDown()) {
                    return;
                }
                if (e.isMetaDown()) {
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
        });
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getTitle() {
        return this.title;
    }

    public int addResult(Result result) {
        if (result.get("hash") != null) {
            //status.setText("<html><font color=red>" + (++hits) + "
            // hits&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</font></html>"
            // );
            status.setText((++hits) + " hits ");
            for (int i = 0; i < model.getRowCount(); i++) {
                if (model.getValueAt(i, 8).equals(result.get("hash"))) {
                    // increase source column
                    int count = ((Integer) model.getValueAt(i, 1)).intValue();
                    count++;
                    model.setValueAt(new Integer(count), i, 1);

                    // increase availability
                    int avail = ((Integer) model.getValueAt(i, 2)).intValue();
                    avail += Integer.parseInt((String) result
                            .get("availability"));
                    model.setValueAt(new Integer(avail), i, 2);

                    // add this source to result
                    Result res = (Result) model.getValueAt(i, 0);
                    res.addSource(result);

                    model.fireTableCellUpdated(i, 1);
                    model.fireTableCellUpdated(i, 2);
                    model.sort();

                    return hits;
                }
            }

            Object bit = null;
            Object duration = null;
            Object resolution = null;
            Object size = null;
            Object availability = null;
            String dur = null;

            Vector subcmds = null;
            Subcommand subcmd = null;
            try {
                subcmds = result.getCommand().getSubCmds();
                if (subcmds.size() <= 0) {
                    return hits;
                }
                subcmd = (Subcommand) subcmds.get(0);

                bit = subcmd.get("bitrate");
                if (bit == null) {
                    bit = "0";
                }
                duration = subcmd.get("duration");
                if (duration == null) {
                    duration = "0";
                }
                resolution = subcmd.get("resolution");
                if (resolution == null) {
                    resolution = "";
                }
                size = result.get("size");
                if (size == null) {
                    size = "0";
                }
                availability = result.get("availability");

                int temp = Integer.parseInt((String) duration);
                int minutes = temp / 60;
                int seconds = temp % 60;
                String separator = ":";
                if (seconds < 10) {
                    separator = ":0";
                }
                dur = minutes + separator + seconds;
            } catch (Exception e) {
                logger.error(e);
            }

            Vector row = new Vector();
            result.addSource(result);
            row.add(result);
            row.add(new Integer(1));
            if (availability == null) {
                availability = "0";
            }
            row.add(new Integer((String) availability));
            row.add(new Long((String) size));
            row.add(new Integer(new Integer((String) bit).intValue() / 1000));
            row.add(dur);
            row.add((String) result.get("mime"));
            row.add(resolution);
            row.add(result.get("hash"));

            model.addRow(row);
        }
        return hits;
    }

    public int getSessionID() {
        return this.sessionID;
    }

    public boolean getEnded() {
        return ended;
    }

    public void setEnded() {
        progress.setIndeterminate(false);
        progress.setMaximum(100);
        progress.setValue(100);
        progress.setString("finished");
        stopButton.setVisible(false);
        ended = true;
    }

    public void triggerDownload(int row) {
        //stcr.setRowRed(row);
        repaint();
        Result main = (Result) model.getValueAt(row, 0);
        String hash = (String) main.get("hash");
        String save = (String) main.toString();
        Vector results = main.getSources();
        Enumeration enumeration = results.elements();
        while (enumeration.hasMoreElements()) {
            Result current = (Result) enumeration.nextElement();
            String user = (String) current.get("user");
            String size = (String) current.get("size");
            String url = (String) current.get("url");
            Main.com.addSource(user, hash, size, url, save);
        }
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == popDownload) {
            int[] rows = table.getSelectedRows();
            for (int i = 0; i < rows.length; i++) {
                triggerDownload(rows[i]);
            }
        } else if (e.getSource() == stopButton) {
            Main.com.cancelSearch(this.sessionID);
        }
    }

}

