package de.hampelratte.yaggui.gui.sortabletable;

import java.util.Collections;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;

public class SortableTableModel extends AbstractTableModel implements Runnable {
    private boolean changed = false;

    private SortableTable table;

    private TableSorter sorter = new TableSorter(this);

    private int cols = 0;

    private Vector rows = new Vector();

    private Vector modelListeners = new Vector();

    public SortableTableModel(int cols, SortableTable table) {
        this.cols = cols;
        this.table = table;
        new Thread(this).start();
    }

    public Object getValueAt(int row, int col) {
        return ((Vector) rows.elementAt(row)).elementAt(col);
    }

    public void setValueAt(Object o, int row, int col) {
        ((Vector) rows.elementAt(row)).setElementAt(o, col);
    }

    public int getColumnCount() {
        return cols;
    }

    public int getRowCount() {
        return rows.size();
    }

    public void addRow(Vector row) {
        /*
         * bubble insert. iterate the list and insert at the adequate position
         * ok for less than 1500 rows. slows down after 1000 rows. fast enough
         * for swing.
         */
        int rowNumber = bubbleInsertRow(row);

        /*
         * binary search for new position very fast. but not working correct at
         * the moment
         */
        //binaryInsertRow(row);
        fireRowAdded(rowNumber);
        //changed = true;
    }

    private int bubbleInsertRow(Vector row) {
        int rowNumber = 0;
        if (table.isSorted()) {
            int col = table.getSortColumn();
            boolean dir = table.getSortDirection();
            if (dir == SortableTable.ASCENDING) {
                boolean inserted = false;
                for (int i = 0; i < rows.size(); i++) {
                    Object o1 = getValueAt(i, col);
                    Object o2 = row.elementAt(col);
                    int result = TableSorter.compare(o2, o1);
                    if (result <= 0) {
                        rows.insertElementAt(row, i);
                        rowNumber = i;
                        inserted = true;
                        break;
                    }
                }
                if (!inserted) {
                    rows.add(row);
                    rowNumber = rows.size() - 1;
                }
            } else {
                boolean inserted = false;
                for (int i = 0; i < rows.size(); i++) {
                    Object o1 = getValueAt(i, col);
                    Object o2 = row.elementAt(col);
                    int result = TableSorter.compare(o1, o2);
                    if (result <= 0) {
                        rows.insertElementAt(row, i);
                        rowNumber = i;
                        inserted = true;
                        break;
                    }
                }
                if (!inserted) {
                    rows.insertElementAt(row, 0);
                    rowNumber = 0;
                }
            }
        } else {
            rows.add(row);
            rowNumber = rows.size() - 1;
        }

        return rowNumber;
    }

    /*
     * private void binaryInsertRow(Vector row) { if(table.isSorted()) { int col =
     * table.getSortColumn(); boolean dir = table.getSortDirection(); boolean
     * inserted = false; int start = 0; int stop = rows.size()-1; int median =
     * 0; int oldmedian = 0; int result = 0; Object o2 = row.elementAt(col);
     * //int count = 0; while(!inserted) { //if(count++ == 50) break; oldmedian =
     * median; //median = (int)Math.round(((double)(start + stop)) / 2.0);
     * median = (start + stop) / 2; if(oldmedian == median) {
     * //System.out.println("fertig " + median); rows.insertElementAt(row,
     * median); break; } //System.out.print("Median:" + median + "
     * start:"+start+" stop:"+stop); Object o1 =
     * ((Vector)rows.elementAt(median)).elementAt(col); if(dir ==
     * SortableTable.ASCENDING) { result = sorter.compare(o1,o2); } else {
     * result = sorter.compare(o2,o1); } //System.out.println(" result:" +
     * result); if(result < 0) { // new row is greater than the current start =
     * median; } else if(result > 0) { stop = median; } } } else {
     * rows.add(row); } }
     */

    private void fireRowAdded(int row) {
        Iterator it = this.modelListeners.iterator();
        while (it.hasNext()) {
            TableModelListener tml = (TableModelListener) it.next();
            //tml.tableChanged( new TableModelEvent(this) );
            //tml.tableChanged( new TableModelEvent(this,
            // TableModelEvent.HEADER_ROW) );
            tml.tableChanged(new TableModelEvent(this, row, row,
                    TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
        }
    }

    public void addTableModelListener(TableModelListener tml) {
        this.modelListeners.add(tml);
    }

    public void removeTableModelListener(TableModelListener tml) {
        this.modelListeners.remove(tml);
    }

    public Vector getData() {
        return this.rows;
    }

    public void run() {
        while (true) {
            try {
                if (changed) {
                    //this.fireRowAdded();
                    changed = false;
                }
                Thread.sleep(1000);
            } catch (Exception e) {
                System.err.println(e);
            }
        }
    }

    public void setRowCount(int count) {
        while (rows.size() > count) {
            rows.remove(rows.lastElement());
        }
    }

    public void removeRow(int i) {
        rows.remove(i);
    }

    public void setColumnIdentifiers(Object[] ident) {
        for (int i = 0; i < ident.length; i++) {
            table.getColumnModel().getColumn(i).setHeaderValue(ident[i]);
        }
    }

    public void sort() {
        // store and clear selection
        int[] indices = table.getSelectedRows();
        Vector selectedRows = new Vector();
        for (int i = 0; i < indices.length; i++) {
            selectedRows.add(rows.elementAt(indices[i]));
        }
        table.clearSelection();

        // sort table
        Collections.sort(rows, new RowComparator(table.getSortColumn(), table
                .getSortDirection()));

        // restore selection
        Iterator it = selectedRows.iterator();
        while (it.hasNext()) {
            Vector row = (Vector) it.next();
            for (int i = 0; i < rows.size(); i++) {
                Vector current = (Vector) rows.elementAt(i);
                if (row == current) {
                    table.addRowSelectionInterval(i, i);
                }
            }
        }
    }
}