package de.hampelratte.yaggui.gui.treetable;

import java.text.DecimalFormat;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import de.hampelratte.yaggui.Load;
import de.hampelratte.yaggui.Logger;
import de.hampelratte.yaggui.Source;

/**
 * 
 * @author hampelratte@users.sf.net
 * 
 * TODO: correct TableModelEvents in the whole model
 */
public class TreeTableModel extends AbstractTableModel {
    private Logger logger = Logger.getLogger();

    private TreeTable table;

    private Vector root = new Vector();

    private int cols;

    public TreeTableModel(TreeTable table, int cols) {
        this.table = table;
        this.cols = cols;
    }

    public Vector getRowAt(int row) {
        if (row > getRowCount()) {
            return null;
        }

        int rowCount = -1;
        Iterator it = root.iterator();
        while (it.hasNext()) {
            rowCount++;
            TreeTableTreeNode node = (TreeTableTreeNode) it.next();
            if (rowCount == row) {
                return node.getData();
            } else {
                if (!node.isCollapsed()) {
                    if ((node.getChildCount() + rowCount) < row) {
                        rowCount += node.getChildCount();
                        continue;
                    } else {
                        for (int i = 0; i < node.getChildCount(); i++) {
                            rowCount++;
                            if (rowCount == row) {
                                return node.getChildAt(i);
                            }
                        }
                    }
                }
            }
        }
        return null;
    }

    public TreeTableTreeNode getTreeNodeAt(int row) {
        int rowCount = -1;
        Iterator it = root.iterator();
        while (it.hasNext()) {
            rowCount++;
            TreeTableTreeNode node = (TreeTableTreeNode) it.next();
            if (rowCount == row) {
                return node;
            } else {
                if (!node.isCollapsed()) {
                    if ((node.getChildCount() + rowCount) < row) {
                        rowCount += node.getChildCount();
                        continue;
                    } else {
                        for (int i = 0; i < node.getChildCount(); i++) {
                            rowCount++;
                            if (rowCount == row) {
                                return node;
                            }
                        }
                    }
                }
            }
        }
        return null;
    }

    public Object getValueAt(int row, int col) {
        Object value = null;
        try {
            Vector _row = getRowAt(row);
            if (_row == null) {
                return null;
            }
            value = getRowAt(row).get(col);
        } catch (Exception e) {
            logger.error(e);
        }
        return value;
    }

    public int getColumnCount() {
        return this.cols;
    }

    public int getRowCount() {
        int rowCounter = 0;
        rowCounter = 0;
        rowCounter += root.size();
        Iterator it = root.iterator();
        while (it.hasNext()) {
            TreeTableTreeNode node = (TreeTableTreeNode) it.next();
            if (!node.isCollapsed()) {
                rowCounter += node.getChildCount();
            }
        }
        return rowCounter;
    }

    private int getPosition(TreeTableTreeNode node) {
        int counter = -1;
        for (int i = 0; i < root.size(); i++) {
            counter++;
            TreeTableTreeNode current = (TreeTableTreeNode) root.elementAt(i);

            if (node == current) {
                break;
            }

            if (!current.isCollapsed()) {
                counter += current.getChildCount();
            }
        }
        return counter;
    }

    public void addRow(Vector row) {
        TreeTableTreeNode node = new TreeTableTreeNode(row);
        root.add(node);
        int _row = getPosition(node);
        super.fireTableRowsInserted(_row, _row);
    }

    /**
     * Child rows are not counted. So you have to set row, as if all rows are
     * collapsed.
     */
    public void addChildAt(Vector child, int row) {
        TreeTableTreeNode node = (TreeTableTreeNode) root.elementAt(row);
        node.addChild(child);
        if (!node.isCollapsed()) {
            int firstRow = getPosition(node);
            super.fireTableRowsInserted(firstRow + node.getChildCount(),
                    firstRow + node.getChildCount());
        }
    }

    public void removeTreeNode(TreeTableTreeNode node) {
        root.remove(node);
    }

    public boolean isChild(int row) {
        int rowCount = -1;
        Iterator it = root.iterator();
        while (it.hasNext()) {
            rowCount++;
            TreeTableTreeNode node = (TreeTableTreeNode) it.next();
            if (rowCount == row) {
                return false;
            } else {
                if (!node.isCollapsed()) {
                    if ((node.getChildCount() + rowCount) < row) {
                        rowCount += node.getChildCount();
                        continue;
                    } else {
                        for (int i = 0; i < node.getChildCount(); i++) {
                            rowCount++;
                            if (rowCount == row) {
                                return true;
                            }
                        }
                    }
                }
            }
        }

        //throw new Exception("No such row exists");
        return false;
    }

    public boolean isChildOfLastNode(int row) {
        int counter = 0;
        int lastNode = 0;
        for (int i = 0; i < this.getRowCount(); i++) {
            if (!isChild(i)) {
                lastNode = counter;
            }
            counter++;
        }
        return row > lastNode;
    }

    public boolean isLastChild(int row) {
        int rowCount = -1;
        Iterator it = root.iterator();
        while (it.hasNext()) {
            rowCount++;
            TreeTableTreeNode node = (TreeTableTreeNode) it.next();
            if (rowCount == row) {
                return false;
            } else {
                if (!node.isCollapsed()) {
                    if ((node.getChildCount() + rowCount) < row) {
                        rowCount += node.getChildCount();
                        continue;
                    } else {
                        for (int i = 0; i < node.getChildCount(); i++) {
                            rowCount++;
                            if (rowCount == row) {
                                if (i == node.getChildCount() - 1) {
                                    return true;
                                }
                            }
                        }
                    }
                }
            }
        }

        //throw new Exception("No such row exists");
        return false;
    }

    public boolean isLastNode(int row) {
        if (root.size() > 0) {
            return this.getTreeNodeAt(row) == root.lastElement();
        } else {
            return false;
        }
    }

    /**
     * Expands a TreeTableTreeNode. All childrens will be shown.
     * 
     * @param node
     *            The node to expand
     * @param row
     *            The absolute row of the Node
     */
    public void expandRow(int row) {
        TreeTableTreeNode node = getTreeNodeAt(row);
        node.setCollapsed(false);
        super.fireTableRowsInserted(row + 1, row + node.getChildCount());
    }

    /**
     * Collapses a TreeTableTreeNode. All childrens will be hidden.
     * 
     * @param node
     *            The node to collapse
     * @param row
     *            The absolute row of the Node
     */
    public void collapseRow(int row) {
        TreeTableTreeNode node = getTreeNodeAt(row);
        node.setCollapsed(true);
        super.fireTableRowsDeleted(row + 1, row + node.getChildCount());
    }

    public void toggleRow(int row) {
        if (!isChild(row)) {
            TreeTableTreeNode node = getTreeNodeAt(row);
            if (node.isCollapsed()) {
                expandRow(row);
            } else {
                collapseRow(row);
            }
        }
    }

    public void addLoad(Load l) {
        Vector row = new Vector();
        row.add(l);
        row.add(new Integer(l.getSourceCount()));
        row
                .add(new Integer(calculateProgress(l.getTransmitted(), l
                        .getSize())));
        row.add(l.getState());
        row.add(formatThroughput(calculateThroughput(l.getElapsed(), l
                .getThroughput())));
        row.add(calculateTimeRemaining(l.getSize(), l.getTransmitted(), l
                .getThroughput()));
        row.add(l.getTransmitted() + " / " + l.getSize());
        addRow(row);

        Vector sources = l.getSources();
        Iterator it = sources.iterator();
        while (it.hasNext()) {
            Vector child = new Vector();
            Source source = (Source) it.next();
            child.add(source);
            child.add(source.getUser());
            child.add(new Integer(calculateProgress(source.getTransmitted(),
                    source.getTotal())));
            child.add(source.getState());
            child.add(formatThroughput(calculateThroughput(source.getElapsed(),
                    source.getThroughput())));
            child.add(calculateTimeRemaining(source.getTotal(), source
                    .getTransmitted(), calculateThroughput(source.getElapsed(),
                    source.getThroughput())));
            child.add(source.getTransmitted() + " / " + source.getTotal());

            this.addChildAt(child, root.size() - 1);
        }
    }

    public void updateLoad(Load l) {
        for (int i = 0; i < root.size(); i++) {
            TreeTableTreeNode node = (TreeTableTreeNode) root.elementAt(i);
            Vector row = node.getData();
            Load load = (Load) row.elementAt(0);

            if (load.getSession() == l.getSession()) {
                row.setElementAt(l, 0);
                row.setElementAt(new Integer(l.getSourceCount()), 1);
                row.setElementAt(new Integer(calculateProgress(l
                        .getTransmitted(), l.getSize())), 2);
                row.setElementAt(l.getState(), 3);
                row.setElementAt(formatThroughput(calculateThroughput(l
                        .getElapsed(), l.getThroughput())), 4);
                row.setElementAt(calculateTimeRemaining(l.getSize(), l
                        .getTransmitted(), calculateThroughput(l.getElapsed(),
                        l.getThroughput())), 5);
                row.setElementAt(l.getTransmitted() + " / " + l.getSize(), 6);

                for (int k = 0; k < node.getChildCount(); k++) {
                    Vector childRow = node.getChildAt(k);
                    Source child = (Source) childRow.elementAt(0);
                    for (int j = 0; j < l.getSourceCount(); j++) {
                        Source source = (Source) l.getSource(j);
                        if (child.getUser().equals(source.getUser())) {
                            childRow.setElementAt(source, 0);
                            childRow.setElementAt(source.getUser(), 1);
                            childRow.setElementAt(new Integer(
                                    calculateProgress(source.getTransmitted(),
                                            source.getTotal())), 2);
                            childRow.setElementAt(source.getState(), 3);
                            //calculate throughput
                            int throughput = (int) (source.getTransmitted() - child
                                    .getTransmitted());
                            source.setThroughput(throughput);
                            childRow.setElementAt(
                                    formatThroughput(calculateThroughput(source
                                            .getElapsed(), source
                                            .getThroughput())), 4);
                            childRow.setElementAt(calculateTimeRemaining(source
                                    .getTotal(), source.getTransmitted(),
                                    calculateThroughput(source.getElapsed(),
                                            source.getThroughput())), 5);
                            childRow.setElementAt(source.getTransmitted()
                                    + " / " + source.getTotal(), 6);
                        }
                    }
                }

                int firstRow = getPosition(node);
                if (node.isCollapsed()) {
                    this.fireTableRowsUpdated(firstRow, firstRow);
                } else {
                    this.fireTableRowsUpdated(firstRow, firstRow
                            + node.getChildCount());
                }
            }
        }
    }

    public Vector getLoads() {
        return new Vector();
    }

    public void removeLoad(int session) {
        for (int i = 0; i < root.size(); i++) {
            TreeTableTreeNode node = (TreeTableTreeNode) root.elementAt(i);
            Vector row = node.getData();
            Load l = (Load) row.elementAt(0);
            if (l.getSession() == session) {
                int firstRow = getPosition(node);
                removeTreeNode(node);
                int lastRow = (node.isCollapsed()) ? firstRow : firstRow
                        + node.getChildCount();
                fireTableRowsDeleted(firstRow, lastRow);
                return;
            }
        }
    }

    public void addSource(Source source, int session) {
        // counter to determine the right row for the table event
        int rowCount = 0;
        for (int i = 0; i < root.size(); i++) {
            TreeTableTreeNode node = (TreeTableTreeNode) root.elementAt(i);
            Vector row = node.getData();
            Load load = (Load) row.elementAt(0);
            rowCount++;
            if (!node.isCollapsed()) {
                rowCount += node.getChildCount();
            }

            if (load.getSession() == session) {
                source.setParent(load);
                Vector child = new Vector();
                child.add(source);
                child.add(source.getUser());
                child.add(new Integer(calculateProgress(
                        source.getTransmitted(), source.getTotal())));
                child.add(source.getState());
                child.add(formatThroughput(calculateThroughput(source
                        .getElapsed(), source.getThroughput())));
                child.add(calculateTimeRemaining(source.getTotal(), source
                        .getTransmitted(), calculateThroughput(source
                        .getElapsed(), source.getThroughput())));
                child.add(source.getTransmitted() + " / " + source.getTotal());

                addChildAt(child, i);

                // update the parent user count
                int firstRow = getPosition(node);
                Integer userCount = (Integer) row.elementAt(1);
                int count = userCount.intValue();
                count++;
                row.setElementAt(new Integer(count), 1);
                fireTableRowsUpdated(firstRow, firstRow);

                return;
            }
        }
    }

    public void removeSource(Source src, int session) {
        // counter to determine the right row for the table event
        int rowCount = 0;
        for (int i = 0; i < root.size(); i++) {
            TreeTableTreeNode node = (TreeTableTreeNode) root.elementAt(i);
            Vector row = node.getData();
            Load load = (Load) row.elementAt(0);
            rowCount++;
            if (!node.isCollapsed()) {
                rowCount += node.getChildCount();
            }

            if (load.getSession() == session) {
                for (int j = 0; j < node.getChildCount(); j++) {
                    Vector r = node.getChildAt(j);
                    Source source = (Source) r.elementAt(0);
                    if (source.getUser().equals(src.getUser())) {
                        node.removeChildAt(j);
                        int firstRow = getPosition(node);
                        if (!node.isCollapsed()) {
                            fireTableRowsDeleted(firstRow + j, firstRow + j);
                        }

                        // update the parent user count
                        Integer userCount = (Integer) row.elementAt(1);
                        int count = userCount.intValue();
                        count--;
                        row.setElementAt(new Integer(count), 1);
                        fireTableRowsUpdated(firstRow, firstRow);
                    }
                }
            }
        }
    }

    public void removeAll() {
        root.removeAllElements();
    }

    private int calculateProgress(long transmited, long size) {
        return (int) ((double) transmited / (double) size * 100);
    }

    private double calculateThroughput(int elapsed, int throughput) {
        double rate = (double) throughput / ((double) elapsed / 1000.0)
                / 1024.0;
        return rate;
    }

    private String formatThroughput(double rate) {
        String rateString = new DecimalFormat("0.00 kb/s").format(rate);
        if (rateString.charAt(0) == 65533)
            rateString = "";
        return rateString;
    }

    private String calculateTimeRemaining(long total, long transmitted,
            double rate) {
        double rest = (double) ((total - transmitted) / 1024);
        double secs = rest / rate;
        int seconds = (int) (secs % 60);
        int minutes = (int) (secs / 60);

        String time = "";

        if (minutes < 60) {
            time = minutes + ":"
                    + (seconds >= 10 ? "" + seconds : "0" + seconds) + " mins";
        } else {
            int min = (int) (minutes % 60);
            int hours = (int) (minutes / 60);
            time = min + ":" + (min >= 10 ? "" + min : "0" + min) + " h";
        }
        return time;
    }
}